#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
from PyQt5 import QtWidgets
from giusas.controller import Controller


app = None


def main():
    global app
    app = QtWidgets.QApplication(sys.argv)
    app.setOrganizationName('Dubble')
    app.setOrganizationDomain('esrf.eu')
    app.setApplicationName('GiuSAS')
    ctrl = Controller()
    ctrl.start()
    sys.exit(app.exec())


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

import json
from PyQt5 import QtCore, QtWidgets
import pyqtgraph as pg
from pyqtgraph import dockarea
from .ui.wgiusas import Ui_WGiuSAS
from .imageview import ImageView


class WGiuSAS(QtWidgets.QMainWindow, Ui_WGiuSAS):
    closeEventSignal = QtCore.pyqtSignal()

    def __init__(self, wparams, wftree):
        super().__init__()
        self.wparams = wparams
        self.wftree = wftree
        self.setUI()

    def setUI(self):
        self.setupUi(self)
        self.docks = dockarea.DockArea()
        self.setCentralWidget(self.docks)
        dock2D = dockarea.Dock('2D')
        self.dockqy = dockarea.Dock('qy')
        self.dockqz = dockarea.Dock('qz')
        self.docks.addDock(dock2D, 'left')
        self.docks.addDock(self.dockqy, 'bottom', dock2D)
        self.docks.addDock(self.dockqz, 'right', dock2D)
        self.toolbar2D = QtWidgets.QToolBar(dock2D)
        self.toolbar2D.addActions(
            (self.actionAdjustLevels, self.actionRotateImage, self.actionFlipImageH, self.actionFlipImageV,
             self.actionLogarithmicScale))
        self.plot2DView = ImageView()
        dock2D.addWidget(self.toolbar2D)
        dock2D.addWidget(self.plot2DView)
        self.plotqy = pg.PlotWidget(self.dockqy)
        self.hroi = self.plotqy.plot()
        self.dockqy.addWidget(self.plotqy)
        self.plotqz = pg.PlotWidget(self.dockqz)
        self.vroi = self.plotqz.plot()
        self.dockqz.addWidget(self.plotqz)
        self.dockParams.setWidget(self.wparams)
        self.dockFiles.setWidget(self.wftree)

    def saveSettings(self):
        s = QtCore.QSettings()
        s.setValue('WGiuSAS/Geometry', self.saveGeometry())
        s.setValue('WGiuSAS/State', self.saveState())
        self.dockqz.label.setText('qz')
        self.dockqy.label.setText('qy')
        s.setValue('WGiuSAS/dockState', json.dumps(self.docks.saveState()))
        s.setValue('WGiuSAS/version', 1)

    def loadSettings(self):
        s = QtCore.QSettings()
        self.restoreGeometry(s.value('WGiuSAS/Geometry', b''))
        self.restoreState(s.value('WGiuSAS/State', b''))
        version = s.value('WGiuSAS/version', 0, int)
        if version < 1:
            return
        dockState = s.value('WGiuSAS/dockState', '', str)
        if dockState:
            # noinspection PyBroadException
            try:
                self.docks.restoreState(json.loads(dockState))
            except Exception:  # stupid!
                pass

    def closeEvent(self, event):
        self.closeEventSignal.emit()
        event.accept()
        super(WGiuSAS, self).closeEvent(event)

    def show2d(self, array):
        adjust = self.plot2DView.image is None
        self.plot2DView.setImage(array, autoLevels=False, autoRange=False)
        if adjust:
            self.on_actionAdjustLevels_triggered()

    def showError(self, msg):
        # noinspection PyCallByClass,PyTypeChecker,PyArgumentList
        QtWidgets.QMessageBox.critical(self, 'GiuSAS error', msg)

    @QtCore.pyqtSlot()
    def on_actionAdjustLevels_triggered(self):
        if self.plot2DView.image is not None:
            self.plot2DView.autoRange()
            self.plot2DView.autoLevels()

    def plotHROIData(self, x, y):
        self.hroi.setData(x, y)

    def plotVROIData(self, x, y):
        self.vroi.setData(y, x)

    def changeCutUnits(self, bottom, right):
        self.dockqy.label.setText(bottom)
        self.dockqz.label.setText(right)

    @QtCore.pyqtSlot()
    def on_actionQuit_triggered(self):
        self.close()

    @QtCore.pyqtSlot(bool)
    def on_actionFiles_toggled(self, checked):
        self.dockFiles.setVisible(checked)

    def on_dockFiles_visibilityChanged(self, visible):
        self.actionFiles.setChecked(visible)

    @QtCore.pyqtSlot(bool)
    def on_actionParameters_toggled(self, checked):
        self.dockParams.setVisible(checked)

    def on_dockParams_visibilityChanged(self, visible):
        self.actionParameters.setChecked(visible)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import pickle
from PyQt5 import QtCore, QtWidgets
from pyqtgraph.parametertree import ParameterTree, Parameter


default_params = [
    {'name': 'Beamline parameters', 'type': 'group', 'children': [
        {'name': 'mode', 'type': 'list', 'values': ['GISAXS', 'GIWAXS'], 'value': 'GISAXS'},
        {'name': 'lambda_', 'type': 'float', 'value': 1, 'suffix': u' Å', 'step': 0.1, 'title': 'Wavelength'},
        {'name': 'dist', 'type': 'float', 'value': 0.1, 'suffix': 'm', 'step': 0.1, 'siPrefix': True,
         'title': 'Distance'},
        {'name': 'x0', 'type': 'float', 'value': 100, 'suffix': ' px', 'step': 1},
        {'name': 'y0', 'type': 'float', 'value': 100, 'suffix': ' px', 'step': 1},
        {'name': 'alphai', 'type': 'float', 'value': 0, 'suffix': u'°', 'step': 0.1, 'title': u'αi'},
        {'name': 'pixel', 'type': 'float', 'value': 5e-5, 'suffix': 'm', 'step': 1e-6, 'siPrefix': True,
         'title': 'Pixel size'},
        {'name': 'qp', 'type': 'float', 'value': 0.99, 'title': 'Polarization', }
    ],
     },
    {'name': 'Sample parameters', 'type': 'group', 'children': [
        {'name': 'calibration', 'type': 'float', 'value': 1, 'step': 1e1, 'title': 'Calibration'},
        {'name': 'bkgCoef', 'type': 'float', 'value': 1, 'step': 0.1, 'title': 'Background coefficient'},
        {'name': 'thickness', 'type': 'float', 'value': 1, 'suffix': 'm', 'step': 0.1, 'siPrefix': True,
         'title': 'Thickness'},
        {'name': 'concentration', 'type': 'float', 'value': 0, 'suffix': ' g/l', 'step': 0.1, 'title': 'Concentration'},
    ],
     },
    {'name': 'Cut parameters', 'type': 'group', 'children': [
        {'name': 'cutunits', 'title': 'Cut units', 'type': 'list',
         'values': ['pixels', 'calibrated pixels', 'q', 'deg'], 'value': 'deg'},
        {'name': 'vcutsize', 'title': 'Vertical cut size', 'suffix': ' px', 'step': 1, 'type': 'int', 'value': 1,
         'limits': (1, 10000)},
        {'name': 'hcutsize', 'title': 'Horizontal cut size', 'suffix': ' px', 'step': 1, 'type': 'int', 'value': 1,
         'limits': (1, 10000)},
    ]
     }
]


class Params(QtWidgets.QWidget):
    def __init__(self):
        super().__init__()
        self.paramtree = ParameterTree()
        layout = QtWidgets.QGridLayout()
        self.setLayout(layout)
        layout.addWidget(self.paramtree)
        self.setWindowTitle('GiuSAS parameters')

    def unpickleParams(self, params):
        if params:
            try:
                return pickle.loads(params)
            except ValueError:  # different pickle versions (usually, when run py2 after py3)
                pass
        return None

    def countParams(self, children):
        n = len(children)
        for c in children:
            n += self.countParams(c.children())
        return n

    def countDefaultParams(self, children):
        n = 0
        if isinstance(children, list):
            n += len(children)
            for c in children:
                n += self.countDefaultParams(c)
        elif isinstance(children, dict):
            if 'children' in children:
                for c in children['children']:
                    n += self.countDefaultParams(c)
            else:
                n += 1
        return n

    def loadSettings(self):
        s = QtCore.QSettings()
        self.restoreGeometry(s.value('WParams/Geometry', b''))
        params = self.unpickleParams(s.value('WParams/params', None))
        if params:
            self.params = Parameter.create(name='params', type='group')
            self.params.restoreState(params)
            if self.countParams(self.params.children()) != self.countDefaultParams(default_params):
                self.params = Parameter.create(name='params', type='group', children=default_params)
        else:
            self.params = Parameter.create(name='params', type='group', children=default_params)
        self.paramtree.setParameters(self.params, showTop=False)
        self.paramtree.setWindowTitle('GiuSAS parameters')

    def saveSettings(self):
        s = QtCore.QSettings()
        s.setValue('WParams/Geometry', self.saveGeometry())
        s.setValue('WParams/params', QtCore.QByteArray(pickle.dumps(self.params.saveState(), pickle.HIGHEST_PROTOCOL)))

    def setBeamCenter(self, x, y):
        for group in self.params:
            for child in group:
                if child.name() == 'x0':
                    child.setValue(x)
                if child.name() == 'y0':
                    child.setValue(y)



#!/usr/bin/python
# -*- coding: utf-8 -*-

import numpy as np
from PyQt5 import QtCore, QtGui, QtWidgets
import pyqtgraph as pg
from pyqtgraph.graphicsItems.ViewBox import ViewBoxMenu, ViewBox


class CustomViewBoxMenu(ViewBoxMenu.ViewBoxMenu):
    menuPositionSignal = QtCore.pyqtSignal(QtCore.QPoint)

    def __init__(self, view):
        super().__init__(view)
        self.setCustomActions()

    def setCustomActions(self):
        self.addSeparator()
        self.setBeamCenterAction = self.addAction('Set beam center')
        self.setBeamCenterAction.triggered.connect(self.setBeamCenter)

    def setBeamCenter(self):
        self.menuPositionSignal.emit(self.pos())


class CustomViewBox(ViewBox):
    mouseLeftClickedSignal = QtCore.pyqtSignal()

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.menu = CustomViewBoxMenu(self)
        self.connectSignals()

    def connectSignals(self):
        pass

    def mouseClickEvent(self, event):
        if event.button() == QtCore.Qt.LeftButton:
            self.mouseLeftClickedSignal.emit()
        else:
            ViewBox.mouseClickEvent(self, event)


class AxisItem(pg.AxisItem):
    changeRangeSignal = QtCore.pyqtSignal(float, float)

    def __init__(self, *args, **kwargs):
        self.range = [0, 1]
        super().__init__(*args, **kwargs)
        self.real_range = [0, 1]

    def setRange(self, mn, mx):
        self.real_range = [mn, mx]
        self.changeRangeSignal.emit(mn, mx)

    def setRealRange(self, mn, mx):
        return super().setRange(mn, mx)

    def changeUnits(self):
        self.setRange(*self.real_range)


class ImageView(pg.GraphicsLayoutWidget):
    CROSS_WIDTH = 2
    CROSS_HEIGHT = 30
    CROSS_HALF = CROSS_HEIGHT / 2 - 1
    setBeamCenterSignal = QtCore.pyqtSignal(float, float)
    setHROIDataSignal = QtCore.pyqtSignal(np.ndarray, np.ndarray, bool, tuple)
    setVROIDataSignal = QtCore.pyqtSignal(np.ndarray, np.ndarray, bool, tuple)

    def __init__(self, parent=None, **kwargs):
        super().__init__(parent, **kwargs)
        self.image = None
        self.vsize = 1
        self.hsize = 1
        self.x = 0
        self.y = 0
        self.view = CustomViewBox()
        self.view.setAspectLocked(True)
        self.view.menu.menuPositionSignal.connect(self.calculateBeamCenter)
        self.view.mouseLeftClickedSignal.connect(self.drawROIcuts)
        # TODO: .addPlot adds a PlotItem, which takes AxisItem parameters
        # TODO: this AxisItems should be considered to show angles from model (or momentum)
        self.leftAxis = AxisItem('left')
        self.bottomAxis = AxisItem('bottom')
        axisItems = {'left': self.leftAxis, 'bottom': self.bottomAxis}
        self.plot2D = self.addPlot(viewBox=self.view, axisItems=axisItems)
        self.imageItem = pg.ImageItem()
        self.plot2D.addItem(self.imageItem)
        self.histogram = pg.HistogramLUTItem()
        self.histogram.setImageItem(self.imageItem)
        self.addItem(self.histogram)
        self.drawCross()
        self.drawCrossHair()

    def setImage(self, image, autoLevels=False, autoRange=False):
        self.image = image
        if self.image is None:
            self.imageItem.clear()
        else:
            self.imageItem.setImage(self.image, autoLevels=autoLevels, autoRange=autoRange)

    def setLevels(self, lmin, lmax):
        self.histogram.setLevels(lmin, lmax)

    def autoLevels(self):
        self.setLevels(0, self.image.mean() + 8 * self.image.std())

    def autoRange(self):
        self.view.autoRange()

    def drawCross(self):
        color = QtGui.QColor(255, 0, 0, 150)
        self.vcross = QtWidgets.QGraphicsRectItem(self.CROSS_HALF, 0, self.CROSS_WIDTH, self.CROSS_HEIGHT)
        self.vcross.setPen(QtGui.QPen(color))
        self.vcross.setBrush(QtGui.QBrush(color))
        self.view.addItem(self.vcross)
        self.hcross = QtWidgets.QGraphicsRectItem(0, self.CROSS_HALF, self.CROSS_HEIGHT, self.CROSS_WIDTH)
        self.hcross.setPen(QtGui.QPen(color))
        self.hcross.setBrush(QtGui.QBrush(color))
        self.view.addItem(self.hcross)
        self.vcross.setVisible(False)
        self.hcross.setVisible(False)

    def calculateBeamCenter(self, menuPos):
        pos = self.view.mapSceneToView(self.mapFromGlobal(menuPos))
        x, y = pos.x() - 0.5, pos.y() - 0.5
        self.drawBeamCenter(x, y)
        self.setBeamCenterSignal.emit(x, y)

    def drawBeamCenter(self, x, y):
        self.hcross.setY(y - self.CROSS_HALF)
        self.hcross.setX(x - self.CROSS_HALF)
        self.vcross.setX(x - self.CROSS_HALF)
        self.vcross.setY(y - self.CROSS_HALF)
        self.vcross.setVisible(True)
        self.hcross.setVisible(True)

    def drawCrossHair(self):
        self.vLine = pg.InfiniteLine(angle=90, movable=False)
        self.hLine = pg.InfiniteLine(angle=0, movable=False)
        self.view.addItem(self.vLine, ignoreBounds=True)
        self.view.addItem(self.hLine, ignoreBounds=True)
        self.proxy = pg.SignalProxy(self.view.scene().sigMouseMoved, rateLimit=60, slot=self.mouseMoved)

    def mouseMoved(self, event=None):
        if self.image is None:
            return
        self.drawROIcuts(event[0])

    def drawROIcuts(self, pos=None):
        if pos and self.view.sceneBoundingRect().contains(pos):
            mousePoint = self.view.mapSceneToView(pos)
            x, y = mousePoint.x(), mousePoint.y()
            self.vLine.setPos(x)
            self.hLine.setPos(y)
            self.x, self.y = x, y
        else:
            x, y = self.x, self.y
        if x and y:
            (xmin, xmax), (ymin, ymax) = self.view.viewRange()
            hroi = pg.ROI(pos=(xmin, y-self.hsize/2), size=(xmax-xmin, self.hsize), angle=0)
            vroi = pg.ROI(pos=(x-self.vsize/2, ymin), size=(self.vsize, ymax-ymin), angle=0)
            hroi.setVisible(False)
            vroi.setVisible(False)
            self.view.addItem(hroi)
            self.view.addItem(vroi)
            hdata, hcoords = hroi.getArrayRegion(self.image, self.imageItem, returnMappedCoords=True)
            self.setHROIDataSignal.emit(hdata, hcoords, not bool(pos), (x, y))
            vdata, vcoords = vroi.getArrayRegion(self.image, self.imageItem, returnMappedCoords=True)
            self.setVROIDataSignal.emit(vdata, vcoords, not bool(pos), (x, y))
            self.view.removeItem(hroi)
            self.view.removeItem(vroi)

    def changeHCutSize(self, size):
        self.hsize = size

    def changeVCutSize(self, size):
        self.vsize = size



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'giusas/gui/ui/wgiusas.ui'
#
# Created by: PyQt5 UI code generator 5.9
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_WGiuSAS(object):
    def setupUi(self, WGiuSAS):
        WGiuSAS.setObjectName("WGiuSAS")
        WGiuSAS.resize(953, 700)
        WGiuSAS.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        WGiuSAS.setTabShape(QtWidgets.QTabWidget.Rounded)
        self.centralwidget = QtWidgets.QWidget(WGiuSAS)
        self.centralwidget.setObjectName("centralwidget")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.centralwidget)
        self.horizontalLayout.setObjectName("horizontalLayout")
        WGiuSAS.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(WGiuSAS)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 953, 49))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtWidgets.QMenu(self.menubar)
        self.menuFile.setObjectName("menuFile")
        self.menuView = QtWidgets.QMenu(self.menubar)
        self.menuView.setObjectName("menuView")
        WGiuSAS.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(WGiuSAS)
        self.statusbar.setObjectName("statusbar")
        WGiuSAS.setStatusBar(self.statusbar)
        self.dockFiles = QtWidgets.QDockWidget(WGiuSAS)
        self.dockFiles.setObjectName("dockFiles")
        self.dockWidgetContents = QtWidgets.QWidget()
        self.dockWidgetContents.setObjectName("dockWidgetContents")
        self.dockFiles.setWidget(self.dockWidgetContents)
        WGiuSAS.addDockWidget(QtCore.Qt.DockWidgetArea(8), self.dockFiles)
        self.dockParams = QtWidgets.QDockWidget(WGiuSAS)
        self.dockParams.setObjectName("dockParams")
        self.dockWidgetContents_2 = QtWidgets.QWidget()
        self.dockWidgetContents_2.setObjectName("dockWidgetContents_2")
        self.dockParams.setWidget(self.dockWidgetContents_2)
        WGiuSAS.addDockWidget(QtCore.Qt.DockWidgetArea(1), self.dockParams)
        self.actionRotateImage = QtWidgets.QAction(WGiuSAS)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/rotate"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionRotateImage.setIcon(icon)
        self.actionRotateImage.setObjectName("actionRotateImage")
        self.actionFlipImageH = QtWidgets.QAction(WGiuSAS)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/flip-horizontal"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionFlipImageH.setIcon(icon1)
        self.actionFlipImageH.setObjectName("actionFlipImageH")
        self.actionFlipImageV = QtWidgets.QAction(WGiuSAS)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/flip-vertical"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionFlipImageV.setIcon(icon2)
        self.actionFlipImageV.setObjectName("actionFlipImageV")
        self.actionAdjustLevels = QtWidgets.QAction(WGiuSAS)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/adjust"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionAdjustLevels.setIcon(icon3)
        self.actionAdjustLevels.setObjectName("actionAdjustLevels")
        self.actionLogarithmicScale = QtWidgets.QAction(WGiuSAS)
        self.actionLogarithmicScale.setCheckable(True)
        self.actionLogarithmicScale.setObjectName("actionLogarithmicScale")
        self.actionQuit = QtWidgets.QAction(WGiuSAS)
        self.actionQuit.setObjectName("actionQuit")
        self.actionParameters = QtWidgets.QAction(WGiuSAS)
        self.actionParameters.setCheckable(True)
        self.actionParameters.setChecked(True)
        self.actionParameters.setObjectName("actionParameters")
        self.actionFiles = QtWidgets.QAction(WGiuSAS)
        self.actionFiles.setCheckable(True)
        self.actionFiles.setChecked(True)
        self.actionFiles.setObjectName("actionFiles")
        self.menuFile.addAction(self.actionQuit)
        self.menuView.addAction(self.actionParameters)
        self.menuView.addAction(self.actionFiles)
        self.menubar.addAction(self.menuFile.menuAction())
        self.menubar.addAction(self.menuView.menuAction())

        self.retranslateUi(WGiuSAS)
        QtCore.QMetaObject.connectSlotsByName(WGiuSAS)

    def retranslateUi(self, WGiuSAS):
        _translate = QtCore.QCoreApplication.translate
        WGiuSAS.setWindowTitle(_translate("WGiuSAS", "GiuSAS - GISAXS and GIWAXS processing application"))
        self.menuFile.setTitle(_translate("WGiuSAS", "&File"))
        self.menuView.setTitle(_translate("WGiuSAS", "&View"))
        self.actionRotateImage.setText(_translate("WGiuSAS", "Rotate image clockwise"))
        self.actionRotateImage.setToolTip(_translate("WGiuSAS", "Rotate image clockwise"))
        self.actionRotateImage.setShortcut(_translate("WGiuSAS", "Ctrl+R"))
        self.actionFlipImageH.setText(_translate("WGiuSAS", "Flip image horizontally"))
        self.actionFlipImageH.setToolTip(_translate("WGiuSAS", "Flip image horizontally"))
        self.actionFlipImageH.setShortcut(_translate("WGiuSAS", "Ctrl+H"))
        self.actionFlipImageV.setText(_translate("WGiuSAS", "Flip image vertically"))
        self.actionFlipImageV.setToolTip(_translate("WGiuSAS", "Flip image vertically"))
        self.actionFlipImageV.setShortcut(_translate("WGiuSAS", "Ctrl+V"))
        self.actionAdjustLevels.setText(_translate("WGiuSAS", "Adjust levels"))
        self.actionAdjustLevels.setToolTip(_translate("WGiuSAS", "Adjust image levels"))
        self.actionAdjustLevels.setShortcut(_translate("WGiuSAS", "Ctrl+A"))
        self.actionLogarithmicScale.setText(_translate("WGiuSAS", "Log"))
        self.actionLogarithmicScale.setShortcut(_translate("WGiuSAS", "Ctrl+L"))
        self.actionQuit.setText(_translate("WGiuSAS", "Quit"))
        self.actionParameters.setText(_translate("WGiuSAS", "Parameters"))
        self.actionFiles.setText(_translate("WGiuSAS", "Files"))

from . import resources_rc



#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
import copy
import math
import functools
import numpy as np
from PyQt5 import QtCore
import cryio
from decor import background, darkcurrent, floodfield


class GiSASModel(QtCore.QObject):
    T_ROT90, T_ROT180, T_ROT270, T_H_FLIP, T_V_FLIP = 1, 2, 3, 4, 5
    data2dSignal = QtCore.pyqtSignal(object)
    errorSignal = QtCore.pyqtSignal(str)
    cutHValuesSignal = QtCore.pyqtSignal(np.ndarray, np.ndarray)
    cutVValuesSignal = QtCore.pyqtSignal(np.ndarray, np.ndarray)
    saveModelParametersSignal = QtCore.pyqtSignal(dict)
    changeCutUnitsSignal = QtCore.pyqtSignal(str, str)
    changeVCutSizeSignal = QtCore.pyqtSignal(int)
    changeHCutSizeSignal = QtCore.pyqtSignal(int)
    verticalRangeSignal = QtCore.pyqtSignal(float, float)
    horizontalRangeSignal = QtCore.pyqtSignal(float, float)

    def __init__(self, parent=None):
        super().__init__(parent)
        self.setDefaultValues()
        self.dropImages()
        self.transformer = {
            self.T_ROT90: functools.partial(self.transformImages, functools.partial(np.rot90, k=self.T_ROT90)),
            self.T_ROT180: functools.partial(self.transformImages, functools.partial(np.rot90, k=self.T_ROT180)),
            self.T_ROT270: functools.partial(self.transformImages, functools.partial(np.rot90, k=self.T_ROT270)),
            self.T_H_FLIP: functools.partial(self.transformImages, np.flipud),
            self.T_V_FLIP: functools.partial(self.transformImages, np.fliplr),
        }

    def setDefaultValues(self):
        self.mode = 'GISAXS'
        self.cos_thetaf = None
        self.cos_alphaf = None
        self.applyBkg = False
        self.applyDark = False
        self.applyFlood = False
        self.applySpline = False
        self.transmission = 1
        self.calibration = 1
        self.bkgCoef = 1
        self.thickness = 1
        self.concentration = 0
        self.lambda_ = 1
        self.dist = 100
        self.x0 = 100
        self.y0 = 100
        self.alphai = 1
        self.log = False
        self.pixel = 50
        self.vcutsize = 1
        self.hcutsize = 1
        self.cutunits = 'deg'
        self.framepath = ''
        self.framename = ''
        self.qp = 0.99
        self.bkgFiles = []
        self.background = background.Background()
        self.dark = darkcurrent.DarkCurrent()
        self.flood = floodfield.FloodField()

    def switchLog(self, log):
        self.log = log
        self.emitData()

    def dropImages(self):
        self.frame = None
        self.data = None
        self.transformations = []
        self.emitData()

    def average(self, flist):
        i, frame = 0, None
        while frame is None or i < len(flist):
            first = flist[i]
            frame = self.openImage(first)
            i += 1
        for f in flist[i:]:
            image = self.openImage(f)
            if image:
                frame += image
        frame.float()
        frame.array /= len(flist)
        return frame

    def openFrames(self, flist, calculate=True):
        if not flist:
            self.dropImages()
            return
        self.framepath = os.path.dirname(flist[0])
        self.framename = os.path.basename(flist[0])
        self.frameslist = flist
        self.frame = self.average(flist)
        if self.frame and calculate:
            self.calculate()
            self.emitData()

    def openBkg(self, flist, calculate=True):
        if flist:
            self.background.init(flist, dark=self.dark, coefficient=self.bkgCoef)
        else:
            self.background.init(None)
        self.bkgFiles = flist
        if calculate:
            self.calculate()
            self.emitData()

    def openFlood(self, flist, calculate=True):
        if isinstance(flist, (list, tuple)) and flist:
            flist = flist[0]
        if flist:
            self.flood.init(flist)
        else:
            self.flood.init(None)
        if calculate:
            self.calculate()
            self.emitData()

    def openSpline(self, spline, calculate=True):
        pass

    def openDark(self, flist, calculate=True):
        if flist:
            self.dark.init(flist)
            self.openBkg(self.bkgFiles)
        else:
            self.dark.init(None)
            self.openBkg(self.bkgFiles)
        if calculate:
            self.calculate()
            self.emitData()

    def calculate(self):
        if self.frame is not None:
            # we copy the frame because otherwise application of the parameters (dark, bkg, etc) does not work
            frame = copy.deepcopy(self.frame)
            if self.applyDark:
                frame = self.dark(frame)
            if self.applyFlood:
                frame = self.flood(frame)
            if self.applyBkg:
                frame = self.background(frame, True, self.thickness, self.concentration)
            self.data = self.applyTransformations(frame.array * self.calibration)

    def emitData(self):
        data = None
        if self.data is not None:
            data = self.data[:]
            if self.log:
                if self.applyDark or self.applyBkg:
                    data[data <= 0] = data.min() * 1e-10
                data = np.log(self.data)
        self.data2dSignal.emit(data)

    def setBkgState(self, state, calculate=True):
        self.applyBkg = state
        if calculate:
            self.calculate()
            self.emitData()

    def setDarkState(self, state, calculate=True):
        self.applyDark = state
        if calculate:
            self.calculate()
            self.emitData()

    def setFloodState(self, state, calculate=True):
        self.applyFlood = state
        if calculate:
            self.calculate()
            self.emitData()

    def setSplineState(self, state, calculate=True):
        self.applySpline = state
        if calculate:
            self.calculate()
            self.emitData()

    def openImage(self, filepath):
        try:
            frame = cryio.openImage(filepath)
        except (cryio.ImageError, IOError, OSError) as err:
            self.errorSignal.emit(err.strerror)
            frame = None
        return frame

    def setParameters(self, params, calculate=True):
        for param in params:
            if param in self.__dict__:
                getattr(self, 'set{}'.format(param.title()))(params[param])
        if calculate:
            self.calculate()
            self.emitData()

    def setMode(self, mode):
        self.mode = mode

    def setCalibration(self, value):
        self.calibration = value
        self.calculate()
        self.emitData()

    def setCutunits(self, value):
        self.cutunits = value
        if 'pixels' in value:
            self.changeCutUnitsSignal.emit('pixel_x', 'pixel_y')
        elif value == 'q':
            self.changeCutUnitsSignal.emit('q_y', 'q_z')
        elif value == 'deg':
            self.changeCutUnitsSignal.emit('theta_f', 'alpha_f')

    def setHcutsize(self, value):
        self.hcutsize = int(value)
        self.changeHCutSizeSignal.emit(self.hcutsize)

    def setVcutsize(self, value):
        self.vcutsize = int(value)
        self.changeVCutSizeSignal.emit(self.vcutsize)

    def setThickness(self, value):
        # noinspection PyTypeChecker
        if -1e-5 <= value <= 1e-5:
            self.errorSignal.emit('Sample thickness is to small!')
        else:
            self.thickness = value

    def setBkgcoef(self, value):
        self.bkgCoef = value
        self.openBkg(self.bkgFiles)

    def setConcentration(self, value):
        self.concentration = value
        self.calculate()
        self.emitData()

    def setLambda_(self, value):
        self.lambda_ = value
        self.calcConsts()

    def setDist(self, value):
        self.dist = value
        self.calcConsts()

    def setX0(self, value):
        self.x0 = value

    def setY0(self, value):
        self.y0 = value

    def setAlphai(self, value):
        self.alphai = math.radians(value)

    def setPixel(self, value):
        self.pixel = value
        self.calcConsts()

    def setQp(self, value):
        self.qp = value

    def setStartingParameters(self, pimage=None, pchecks=None):
        if pimage:
            for key in pimage:
                getattr(self, 'open{}'.format(key.title()))(pimage[key], calculate=False)
        if pchecks:
            for key in pchecks:
                setattr(self, key, pchecks[key])

    def rotateImage(self):
        if self.transformations and self.transformations[-1] - 3 <= 0:
            self.transformations[-1] += 1
            if self.transformations[-1] >= 4:
                self.transformations.pop()
        else:
            self.transformations.append(self.T_ROT90)
        self.calculate()
        self.emitData()

    def flipImageH(self):
        # if it is the same transformation, as it was the last one, we just cancel the last one
        if self.transformations and self.transformations[-1] == self.T_H_FLIP:
            self.transformations.pop()
        else:
            self.transformations.append(self.T_H_FLIP)
        self.calculate()
        self.emitData()

    def flipImageV(self):
        # if it is the same transformation, as it was the last one, we just cancel the last one
        if self.transformations and self.transformations[-1] == self.T_V_FLIP:
            self.transformations.pop()
        else:
            self.transformations.append(self.T_V_FLIP)
        self.calculate()
        self.emitData()

    def applyTransformations(self, frame):
        for t in self.transformations:
            frame = self.transformer[t](frame)
        return frame

    def transformImages(self, transformation, frame):
        return transformation(frame)

    def setBeamCenter(self, x, y):
        self.setX0(x)
        self.setY0(y)

    def calcHCut(self, data, coords, save=False, center=None):
        if data is None:
            return
        while data.ndim > 1:
            data = data.mean(axis=1)
        while coords.ndim > 2:
            coords = coords[:, :, 0]
        hcd = coords - coords[:, 0, np.newaxis]
        x = np.sqrt((hcd ** 2).sum(axis=0)) + coords[0][0]
        x = self.calcHCutValues(x)
        self.cutHValuesSignal.emit(x, data)
        self.saveCut(save, x, data, center, 'h')

    def calcHCutValues(self, x):
        if self.cutunits == 'calibrated pixels':
            x = x - self.x0
        elif self.cutunits == 'deg':
            x = np.degrees(self.thetaf(x))
        elif self.cutunits == 'q':
            x = self.qy(x)
        return x

    def thetaf(self, x):
        thetaf = np.arctan((x - self.x0) * self.pixdist)
        self.cos_thetaf = np.cos(thetaf) if self.mode == 'GIWAXS' else None
        return thetaf

    def qy(self, x):
        qy = self.qconst * np.sin(self.thetaf(x))
        if (self.mode == 'GIWAXS' and self.cos_alphaf is not None and isinstance(x, np.ndarray)
           and self.cos_alphaf.shape == x.shape):
            return qy * self.cos_alphaf
        else:
            return qy

    def calcVCut(self, data, coords, save=False, center=None):
        if data is None:
            return
        while data.ndim > 1:
            data = data.mean(axis=0)
        while coords.ndim > 2:
            coords = coords[:, 0, :]
        hcd = coords - coords[:, 0, np.newaxis]
        y = np.sqrt((hcd ** 2).sum(axis=0)) + coords[1][0]
        y = self.calcVCutValues(y)
        self.cutVValuesSignal.emit(y, data)
        self.saveCut(save, y, data, center, 'v')

    def calcVCutValues(self, y):
        if self.cutunits == 'calibrated pixels':
            y = y - self.y0
        elif self.cutunits == 'deg':
            # noinspection PyTypeChecker
            y = np.degrees(self.alphaf(y))
        elif self.cutunits == 'q':
            y = self.qz(y)
        return y

    def alphaf(self, y):
        alphaf = np.arctan((y - self.y0) * self.pixdist)
        if self.mode == 'GIWAXS' and self.cos_thetaf is not None and self.cos_thetaf.shape == alphaf.shape:
            alphaf -= self.alphai * self.cos_thetaf
            self.cos_alphaf = np.cos(alphaf)
        else:
            alphaf -= self.alphai
            self.cos_alphaf = None
        return alphaf

    def qz(self, y):
        return self.qconst * (math.sin(self.alphai) + np.sin(self.alphaf(y)))

    def calcConsts(self):
        if self.lambda_ == 0:
            self.errorSignal.emit('Wavelength cannot be zero')
        elif self.dist == 0:
            self.errorSignal.emit('Distance cannot be zero')
        else:
            self.qconst = 2 * math.pi / self.lambda_ * 10
            self.pixdist = self.pixel / self.dist

    def saveSettings(self):
        self.saveModelParametersSignal.emit({'transformations': ' '.join(str(i) for i in self.transformations)})

    def loadSettings(self, params):
        if 'transformations' in params:
            self.transformations = [int(t) for t in params['transformations'].split()]

    def saveCut(self, save, x, y, center, name):
        if not save or not self.framepath:
            return
        x0, y0 = center
        x0 = self.calcHCutValues(x0)
        y0 = self.calcVCutValues(y0)
        d = os.path.join(self.framepath, 'giusas_cuts')
        if not os.path.exists(d):
            os.mkdir(d)
        if self.cutunits == 'q':
            ux, uy = 'qy', 'qz'
        elif self.cutunits == 'deg':
            ux, uy = 'alpha_f', 'theta_f'
        else:
            ux, uy = 'pixel_x', 'pixel_y'
        fn = os.path.join(d, '{}_{}_{}_{:.3f}_{}_{:.3f}.dat'.format(self.framename, name, ux, x0, uy, y0))
        np.savetxt(fn, np.array((x, y)).T, header='{} I'.format(self.cutunits.replace(' ', '_')))

    def calcVRange(self, ymin, ymax):
        self.verticalRangeSignal.emit(self.calcVCutValues(ymin), self.calcVCutValues(ymax))

    def calcHRange(self, xmin, xmax):
        self.horizontalRangeSignal.emit(self.calcHCutValues(xmin), self.calcHCutValues(xmax))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup

setup(
    name='giusas',
    version='2017.10.22',
    description='GISAXS and GIWAXS calculator',
    author='Vadim Dyadkin',
    author_email='dyadkin@gmail.com',
    url='https://soft.snbl.eu/giusas.html',
    license='GPLv3',
    install_requires=[
        'numpy>=1.9',
        'scipy>=0.10.0',
        'pyqtgraph>=0.10.0',
        'cryio>=2016.10.1',
        'decor>=2016.12.1',
    ],
    entry_points={
        'gui_scripts': [
            'giusas=giusas.__init__:main',
        ],
    },
    packages=[
        'giusas',
        'giusas.gui',
        'giusas.gui.ui',
    ],
)



#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
from PyQt5 import QtCore
from .gui.ptree import Params
from .gui.wgiusas import WGiuSAS
from .gui.ftree import FilesTree
from .model import GiSASModel


class Controller(QtCore.QObject):
    sendParameters = QtCore.pyqtSignal(dict, dict)
    calculateSignal = QtCore.pyqtSignal()
    emitDataSignal = QtCore.pyqtSignal()
    loadModelParametersSignal = QtCore.pyqtSignal(dict)

    def __init__(self, parent=None):
        super().__init__(parent)
        self.createModel()
        self.createWindows()
        self.connectSignals()

    def createModel(self):
        self.gmodelThread = QtCore.QThread()
        self.gmodel = GiSASModel()
        self.gmodel.moveToThread(self.gmodelThread)

    def loadSettings(self):
        self.settings = QtCore.QSettings()
        self.loadModelParameters()
        self.wgiusas.loadSettings()
        self.wparams.loadSettings()
        self.wftree.loadSetting()
        self.wftree.workDir = self.workDir = self.settings.value('WGiuSAS/workDir', '', str)
        # self.sendParameters.emit(self.wgiusas.tabs.imageParameters(), self.wgiusas.tabs.checkBoxesState())
        self.calculateSignal.emit()
        self.emitDataSignal.emit()

    def saveSettings(self):
        self.wgiusas.saveSettings()
        self.wparams.saveSettings()
        self.wftree.saveSettings()
        self.settings.setValue('WGiuSAS/workDir', self.workDir)

    def start(self):
        self.loadSettings()
        self.connectParamsSignals()
        self.gmodelThread.start()
        self.wgiusas.show()

    def createWindows(self):
        self.wparams = Params()
        self.wftree = FilesTree()
        self.wgiusas = WGiuSAS(self.wparams, self.wftree)

    def connectSignals(self):
        self.sendParameters.connect(self.gmodel.setStartingParameters)
        self.calculateSignal.connect(self.gmodel.calculate)
        self.emitDataSignal.connect(self.gmodel.emitData)
        self.wgiusas.closeEventSignal.connect(self.closeAll)
        self.wgiusas.actionRotateImage.triggered.connect(self.gmodel.rotateImage)
        self.wgiusas.actionFlipImageH.triggered.connect(self.gmodel.flipImageH)
        self.wgiusas.actionFlipImageV.triggered.connect(self.gmodel.flipImageV)
        self.wgiusas.actionLogarithmicScale.triggered[bool].connect(self.gmodel.switchLog)
        self.wgiusas.plot2DView.setBeamCenterSignal.connect(self.gmodel.setBeamCenter)
        self.wgiusas.plot2DView.setBeamCenterSignal.connect(self.wparams.setBeamCenter)
        self.gmodel.data2dSignal.connect(self.wgiusas.show2d)
        self.gmodel.errorSignal.connect(self.wgiusas.showError)
        self.wgiusas.plot2DView.setHROIDataSignal.connect(self.gmodel.calcHCut)
        self.wgiusas.plot2DView.setVROIDataSignal.connect(self.gmodel.calcVCut)
        self.gmodel.cutHValuesSignal.connect(self.wgiusas.plotHROIData)
        self.gmodel.cutVValuesSignal.connect(self.wgiusas.plotVROIData)
        # noinspection PyUnresolvedReferences
        self.gmodelThread.finished.connect(self.gmodel.saveSettings)
        self.gmodel.saveModelParametersSignal.connect(self.saveModelParameters)
        self.gmodel.changeCutUnitsSignal.connect(self.wgiusas.changeCutUnits)
        self.gmodel.changeCutUnitsSignal.connect(self.wgiusas.plot2DView.leftAxis.changeUnits)
        self.gmodel.changeCutUnitsSignal.connect(self.wgiusas.plot2DView.bottomAxis.changeUnits)
        self.gmodel.changeHCutSizeSignal.connect(self.wgiusas.plot2DView.changeHCutSize)
        self.gmodel.changeVCutSizeSignal.connect(self.wgiusas.plot2DView.changeVCutSize)
        self.loadModelParametersSignal.connect(self.gmodel.loadSettings)
        self.wgiusas.plot2DView.leftAxis.changeRangeSignal.connect(self.gmodel.calcVRange)
        self.wgiusas.plot2DView.bottomAxis.changeRangeSignal.connect(self.gmodel.calcHRange)
        self.gmodel.verticalRangeSignal.connect(self.wgiusas.plot2DView.leftAxis.setRealRange)
        self.gmodel.horizontalRangeSignal.connect(self.wgiusas.plot2DView.bottomAxis.setRealRange)
        self.wftree.openDataSignal.connect(self.setWorkDir)
        self.wftree.openDataSignal.connect(self.gmodel.openFrames)
        self.wftree.openDarkSignal.connect(self.setWorkDir)
        self.wftree.openDarkSignal.connect(self.gmodel.openDark)
        self.wftree.applyDarkSignal.connect(self.gmodel.setDarkState)
        self.wftree.openBkgSignal.connect(self.setWorkDir)
        self.wftree.openBkgSignal.connect(self.gmodel.openBkg)
        self.wftree.applyBkgSignal.connect(self.gmodel.setBkgState)
        self.wftree.openFloodSignal.connect(self.setWorkDir)
        self.wftree.openFloodSignal.connect(self.gmodel.openFlood)
        self.wftree.applyFloodSignal.connect(self.gmodel.setFloodState)
        self.wftree.openSplineSignal.connect(self.setWorkDir)
        self.wftree.openSplineSignal.connect(self.gmodel.openSpline)
        self.wftree.applySplineSignal.connect(self.gmodel.setSplineState)

    def connectParamsSignals(self):
        for group in self.wparams.params.children():
            for child in group.children():
                child.sigValueChanging.connect(lambda p, v: self.gmodel.setParameters({p.name(): v}))
                child.sigValueChanged.connect(lambda p, v: self.gmodel.setParameters({p.name(): v}))
                child.sigValueChanged.emit(child, child.value())

    def setWorkDir(self, flist):
        if flist:
            self.workDir = self.wftree.workDir = os.path.dirname(flist[0])

    def closeAll(self):
        self.saveSettings()
        self.gmodelThread.quit()
        self.gmodelThread.wait()

    def saveModelParameters(self, params):
        for param in params:
            self.settings.setValue(f'Model/{param}', params[param])

    def loadModelParameters(self):
        self.loadModelParametersSignal.emit(
            {k.split('/')[-1]: self.settings.value(k) for k in self.settings.allKeys() if k.startswith('Model/')}
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import json
import uuid
from PyQt5 import QtCore, QtWidgets


class TreeWidget(QtWidgets.QTreeWidget):
    deleteKeySignal = QtCore.pyqtSignal()

    def keyPressEvent(self, event):
        if event.key() == QtCore.Qt.Key_Delete:
            self.deleteKeySignal.emit()
        else:
            super().keyPressEvent(event)


class TreeWidgetItem(QtWidgets.QTreeWidgetItem):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.__uuid = uuid.uuid4()

    def __hash__(self):
        return hash(self.__uuid)


class ItemProxy:
    def __init__(self, openSignal, applySignal):
        self.openSignal = openSignal
        self.applySignal = applySignal
        self.files = []

    def openEmit(self, files=None):
        self.files = files or self.files
        self.openSignal.emit(self.files)

    def removeFile(self, file):
        self.files.remove(file)
        self.openEmit()


class FilesTree(QtWidgets.QWidget):
    openDataSignal = QtCore.pyqtSignal(list)
    applyDataSignal = QtCore.pyqtSignal(bool)
    openBkgSignal = QtCore.pyqtSignal(list)
    applyBkgSignal = QtCore.pyqtSignal(bool)
    openDarkSignal = QtCore.pyqtSignal(list)
    applyDarkSignal = QtCore.pyqtSignal(bool)
    openFloodSignal = QtCore.pyqtSignal(list)
    applyFloodSignal = QtCore.pyqtSignal(bool)
    openSplineSignal = QtCore.pyqtSignal(list)
    applySplineSignal = QtCore.pyqtSignal(bool)

    def __init__(self):
        super().__init__()
        self.workDir = ''
        self.setWindowTitle('Files')
        self.setUI()
        self.connectSignals()

    def setUI(self):
        self.tree = TreeWidget(self)
        layout = QtWidgets.QGridLayout()
        self.setLayout(layout)
        layout.addWidget(self.tree)
        self.tree.setColumnCount(2)
        headerItem = self.tree.headerItem()
        headerItem.setText(0, 'Files')
        headerItem.setText(1, 'Apply?')
        header = self.tree.header()
        header.setStretchLastSection(False)
        header.setSectionResizeMode(QtWidgets.QHeaderView.ResizeToContents)
        self.dataItem = TreeWidgetItem(self.tree)
        self.dataItem.setText(0, 'Data')
        self.bkgItem = TreeWidgetItem(self.tree)
        self.bkgItem.setText(0, 'Background')
        self.bkgItem.setCheckState(1, QtCore.Qt.Unchecked)
        self.darkItem = TreeWidgetItem(self.tree)
        self.darkItem.setText(0, 'Dark Current')
        self.darkItem.setCheckState(1, QtCore.Qt.Unchecked)
        self.floodItem = TreeWidgetItem(self.tree)
        self.floodItem.setText(0, 'Flat field')
        self.floodItem.setCheckState(1, QtCore.Qt.Unchecked)
        self.splineItem = TreeWidgetItem(self.tree)
        self.splineItem.setText(0, 'Spline')
        self.splineItem.setCheckState(1, QtCore.Qt.Unchecked)
        self.itemSignals = {
            self.dataItem:   ItemProxy(self.openDataSignal, self.applyDataSignal),
            self.bkgItem:    ItemProxy(self.openBkgSignal, self.applyBkgSignal),
            self.darkItem:   ItemProxy(self.openDarkSignal, self.applyDarkSignal),
            self.floodItem:  ItemProxy(self.openFloodSignal, self.applyFloodSignal),
            self.splineItem: ItemProxy(self.openSplineSignal, self.applySplineSignal),
        }

    # noinspection PyUnresolvedReferences
    def connectSignals(self):
        self.tree.itemDoubleClicked.connect(self.itemDoubleClicked)
        self.tree.itemClicked.connect(self.itemClicked)
        self.tree.itemChanged.connect(self.itemChanged)
        self.tree.deleteKeySignal.connect(self.deleteItems)

    def deleteItems(self):
        for item in self.tree.selectedItems():
            if item not in self.itemSignals.keys():
                parent = item.parent()
                # noinspection PyTypeChecker
                parentItem = self.itemSignals[parent]
                parentItem.removeFile(item.text(0))
                parent.removeChild(item)

    def itemChanged(self, item, col):
        if col != 1 or item not in self.itemSignals.keys():
            return
        self.itemSignals[item].applySignal.emit(bool(item.checkState(1)))

    def itemDoubleClicked(self, item, col):
        if col or item not in self.itemSignals.keys():
            return
        # noinspection PyCallByClass,PyTypeChecker
        fs = QtWidgets.QFileDialog.getOpenFileNames(self, 'Select {} files'.format(item.text(0)),
                                                    self.workDir, 'Frames (*.cbf *.edf)')[0]
        self.addSubItems(fs, item)

    def addSubItems(self, fs, item):
        item.setExpanded(True)
        if fs:
            fs.sort()
            for f in fs:
                if os.path.exists(f):
                    i = TreeWidgetItem(item)
                    i.setText(0, f)
            self.itemSignals[item].openEmit(fs)

    def itemClicked(self, item, col):
        if col or item in self.itemSignals.keys():
            return

    def loadSetting(self):
        s = QtCore.QSettings()
        self.restoreGeometry(s.value('WFTree/Geometry', b''))
        for item in self.itemSignals.keys():
            names = json.loads(s.value(f'WFTree/{item.text(0)}', '[]', str))
            self.addSubItems(names, item)
            item.setCheckState(1, s.value(f'WFTree/{item.text(0)}_checked', QtCore.Qt.Unchecked, int))

    def saveSettings(self):
        s = QtCore.QSettings()
        s.setValue('WFTree/Geometry', self.saveGeometry())
        for item in self.itemSignals.keys():
            names = [item.child(i).text(0) for i in range(item.childCount())]
            s.setValue(f'WFTree/{item.text(0)}', json.dumps(names))
            s.setValue(f'WFTree/{item.text(0)}_checked', item.checkState(1))
